package co.com.certicamara.servicio;

import org.springframework.stereotype.Service;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.HttpEntity;
import org.springframework.web.client.RestTemplate;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import co.com.certicamara.dto.EntidadSolicitudDto;
import co.com.certicamara.utils.UtilsXML;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;

@Service
public class SoapClienteServicio {

	private final Logger logger = Logger.getLogger(this.getClass());

	@Value("${HOST_TRUSTED}")
	private String host;

	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	private UtilsXML utilsXML;

	public ResponseEntity<String> listarCertificados(EntidadSolicitudDto entidad) throws Exception {
		logger.info("Inicia peticion SOAP para listar certificados en el TrustedX con el nuip: " + entidad.getNuip());
		try {
			String xmlRequestBody = utilsXML.generarSoapCertificados(entidad.getNuip(), entidad.getPassword(),
					entidad.getNuip());
			HttpHeaders headers = new HttpHeaders();
			headers.set("SOAPAction", "Read");
			headers.set("TwsAuthN", "urn:safelayer:tws:policies:authentication:default:firmadigital");
			return enviarPeticion(xmlRequestBody, headers);
		} catch (Exception e) {
			logger.error("error haciendo la peticion al servicio trusted");
			return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Error: " + e.getMessage());
		}
	}

	public ResponseEntity<String> validarUsuario(EntidadSolicitudDto entidad) throws Exception {
		logger.info("Inicia peticion SOAP para validar si exisite el usuario con el nuip: " + entidad.getNuip());
		try {
			String xmlRequestBody = utilsXML.generarSoapValidarUsuario(entidad.getNuip(), entidad.getPassword(),
					entidad.getNuip());
			HttpHeaders headers = new HttpHeaders();
			headers.set("SOAPAction", "Search");
			headers.set("TwsAuthN", "urn:safelayer:tws:policies:authentication:default:firmadigital");
			return enviarPeticion(xmlRequestBody, headers);
		} catch (Exception e) {
			logger.error("error haciendo la peticion al servicio trusted");
			return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Error: " + e.getMessage());
		}
	}

	public ResponseEntity<String> validarAutenticacion(EntidadSolicitudDto entidad) throws Exception {
		logger.info("Inicia peticion SOAP para validar autenticacion al TrustedX con el nuip: " + entidad.getNuip());
		try {
			String xmlRequestBody = utilsXML.generarSoapValidarAutenticacion(entidad.getNuip(), entidad.getPassword());
			HttpHeaders headers = new HttpHeaders();
			headers.set("SOAPAction", "AuthN");
			headers.set("TwsAuthN", "urn:safelayer:tws:policies:authentication:usersconfig");
			return enviarPeticion(xmlRequestBody, headers);
		} catch (Exception e) {
			logger.error("error haciendo la peticion al servicio trusted");
			return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Error: " + e.getMessage());
		}
	}

	private ResponseEntity<String> enviarPeticion(String xmlRequestBody, HttpHeaders headers) throws Exception {
		try {
			
			StringBuilder buf = new StringBuilder();
			buf.append(xmlRequestBody);
			byte[] outs = buf.toString().getBytes();

			headers.set("Content-Type", "text/xml;charset=UTF-8");
			headers.set("Accept-Encoding", "gzip,deflate");
			headers.set("Content-Length", String.valueOf(outs.length));
			headers.set("Connection", "Keep-Alive");

			HttpEntity<String> requestEntity = new HttpEntity<>(xmlRequestBody, headers);
			ResponseEntity<String> responseEntity = restTemplate.exchange(host, HttpMethod.POST, requestEntity,
					String.class);
			return responseEntity;
		} catch (Exception e) {
			logger.error("fallo la consulta al servidor de trustedx " + e);
			throw new Exception("fallo la consulta al servidor de trustedx " + e.getMessage());
		}

	}
}
